texture tex0 : Diffuse < string ResourceName = "Diffuse.tga"; >;

float4x4 WorldMatrix			: World;
float4x4 ViewMatrix;
float4x4 ProjectionMatrix;
float4x4 WorldViewProjectionMatrix;

float4x4 AbsoluteWorldMatrix;

float4 LightDirection;
float4 LightAmbient;
float4 LightColor;
float4 BlendColor;

const float WRAP = 0.5;
const float INTENSITY = 1.3;

sampler2D DiffuseTexture = 
sampler_state 
{
    texture = <tex0>;
    AddressU  = CLAMP;        
    AddressV  = CLAMP;
    AddressW  = CLAMP;
    MIPFILTER = LINEAR;
    MINFILTER = LINEAR;
    MAGFILTER = LINEAR;
};

struct VS_INPUT 
{
   float3 Position : POSITION;
   float2 TexCoord : TEXCOORD0;
   float3 Normal   : NORMAL;
};

struct VS_OUTPUT 
{
   float4 Position :        POSITION;
   float2 TexCoord :        TEXCOORD0;
   float3 LightDirection :  TEXCOORD1;
   float3 Normal :          TEXCOORD2;
};



VS_OUTPUT MapObject_VS( VS_INPUT In )
{
	VS_OUTPUT Out = (VS_OUTPUT)0;
	Out.Position = mul( float4(In.Position.x, In.Position.y + 13, In.Position.z, 1), WorldViewProjectionMatrix );
	Out.Normal = mul( In.Normal, WorldMatrix );
	Out.LightDirection = LightDirection;
	Out.TexCoord = In.TexCoord;

	return Out;
}



float4 MapObject_PS( VS_OUTPUT In ) : COLOR
{
	float4 DiffuseColor = tex2D( DiffuseTexture, In.TexCoord );
	float3 LightDirection = -In.LightDirection;
	float3 Normal = normalize(In.Normal.xyz);
	float NdotL = max(0, dot( Normal, LightDirection ) );
	
	float WrapDiffuse = saturate((NdotL + WRAP) / (1 + WRAP)) * INTENSITY;
	float3 OutColor = DiffuseColor.rgb * BlendColor;
	return float4( OutColor * WrapDiffuse, 1.0 );
}

technique MapObjectShader
{
	pass p0
	{
		VertexShader = compile vs_2_0 MapObject_VS();
		PixelShader = compile ps_2_0 MapObject_PS();
	}
}
