#define BRIGHTNESS 0.04
#define CONTRAST 1.12

const float3 AMBIENT = float3(0.05f, 0.05f, 0.12f);

#define TILE 50

#define X_OFFSET 0.5
#define Z_OFFSET 0.5

#define FOW_SIZE 1024

texture tex0;
texture tex1;
texture tex2;
texture tex3;
texture tex4;
texture tex5; // FOW

float4x4 WorldMatrix		: World; 
float4x4 ViewMatrix			: View; 
float4x4 ProjectionMatrix	: Projection; 
float3	 LightDirection;
float	 vAlpha;
float	 vTime = 0;
float    vHeight = 0;
float    vFOW = 0.0;
float3	CountryColor1;
float3	CountryColor2;

sampler BaseTexture  =
sampler_state
{
    Texture = <tex0>;
    MinFilter = Linear; //Point;
    MagFilter = Linear; //Point;
    MipFilter = Linear; //None;
    AddressU = Wrap;
    AddressV = Wrap;
};

sampler WavesTexture  =
sampler_state
{
    Texture = <tex0>;
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = None;
    AddressU = Wrap;
    AddressV = Wrap;
};

sampler SlopeTexture  =
sampler_state
{
    Texture = <tex1>;
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = Linear;
    AddressU = Wrap;
    AddressV = Wrap;
};


sampler MapTexture  =
sampler_state
{
    Texture = <tex1>;
    MinFilter = Linear; //Point;
    MagFilter = Linear; //Point;
    MipFilter = Linear; //None;
    AddressU = Wrap;
    AddressV = Wrap;
};

sampler StripesTexture  =
sampler_state
{
    Texture = <tex1>;
    MinFilter = Linear; //Point;
    MagFilter = Linear; //Point;
    MipFilter = None;
    AddressU = Wrap;
    AddressV = Wrap;
};


sampler ColorTexture  =
sampler_state
{
    Texture = <tex4>;
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = None;
    AddressU = Clamp;
    AddressV = Clamp;
};

// used for borders
sampler GeneralTexture  =
sampler_state
{
    Texture = <tex2>;
    MinFilter = Point;
    MagFilter = Point;
    MipFilter = None;
    AddressU = Clamp;
    AddressV = Clamp;
};

sampler GeneralTexture2  =
sampler_state
{
    Texture = <tex3>;
    MinFilter = Point;
    MagFilter = Point;
    MipFilter = None;
    AddressU = Clamp;
    AddressV = Clamp;
};

sampler BorderTexture  =
sampler_state
{
    Texture = <tex0>;
	MipFilter = Linear;
	MinFilter = Anisotropic;
    MaxAnisotropy = 4;
    MagFilter = Linear;
    
    AddressU = Clamp;
    AddressV = Clamp;
};

sampler TreeTexture  =
sampler_state
{
    Texture = <tex0>;
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = Point;
    AddressU = Wrap;
    AddressV = Wrap;
};

sampler FOWTexture  =
sampler_state
{
    Texture = <tex5>;
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = None;
    AddressU = Clamp;
    AddressV = Clamp;
};

//////////////////////////////////////////////////////////

struct VS_INPUT
{
    float3 vPosition  : POSITION;
    float4 vNormal    : NORMAL;
};

struct VS_OUTPUT
{
    float4  vPosition : POSITION;
	float3  vNormal : TEXCOORD0;
	float3  vLightDirection : TEXCOORD4;
    float3  vTexCoord0 : TEXCOORD5;	// x bein' darkening for shores,  z beein' lightIntensity
    float2  vColorTexCoord : TEXCOORD1;
    float3  vTerrainTexCoord : TEXCOORD2;
	float2 vFOWCoords : TEXCOORD6;
};

struct VS_MAP_INPUT
{
    float3 vPosition  : POSITION;
    float4 vNormal    : NORMAL;
    float2 vSmallTextureTexCoord  : TEXCOORD1;
};


struct VS_INPUT_WAVES
{
    float3 vPosition  : POSITION;
    float2 vWaveData  : TEXCOORD0;
};

struct VS_OUTPUT_WAVES
{
    float4  vPosition : POSITION;
    float2  vWaveData : TEXCOORD0;
    float2  vWaveUV	  : TEXCOORD1;
	float2 vFOWCoords : TEXCOORD2;
};

struct VS_INPUT_PTI
{
    float4 vPosition  : POSITION;
};

struct VS_OUTPUT_PTI
{
    float4  vPosition : POSITION;
};

struct VS_BORDER_INPUT
{
    float3 vPosition  : POSITION;
    float2 vBorderIndex : TEXCOORD0;
    float2 vStrength	: TEXCOORD1;
    float2 vCorner	: TEXCOORD2;
};

struct VS_BORDER_OUTPUT
{
    float4 vPosition  : POSITION;
    float2 vBorderIndex : TEXCOORD0;
    float2 vStrength	: TEXCOORD1;
    float2 vCorner	: TEXCOORD2;
	float2 vFOWCoords : TEXCOORD3;
};

struct VS_TREE_INPUT
{
    float3 vPosition  : POSITION;
    float4 vNormal    : NORMAL;
    float2 vTexCoord0 : TEXCOORD0;
};

struct VS_TREE_OUTPUT
{
    float4 vPosition : POSITION;
	float2 vTexCoord0 : TEXCOORD0;
	float  Intensity : TEXCOORD1;
	float2 vFOWCoords : TEXCOORD2;
};

/////////////////////////////////////////////////////////////////////////////////

float3 ApplyFOWColor( float3 c, float FOW ) 
{
	float Grey = dot( c.rgb, float3( 0.212671f, 0.715160f, 0.072169f ) );
	return lerp( c.rgb, Grey.rrr, FOW * 0.4 ) * (1.0 - FOW * 0.35);
}

/////////////////////////////////////////////////////////////////////////////////

VS_OUTPUT VertexShader_Map_General(const VS_INPUT v )
{
	VS_OUTPUT Out = (VS_OUTPUT)0;

	float4 Position = float4( v.vPosition, 1 );

	float4x4 WorldView = ViewMatrix; // mul(WorldMatrix, ViewMatrix);
	float3 P = mul( Position, (float4x3)WorldView);
	Out.vPosition  = mul(float4(P, 1), ProjectionMatrix);
	float3 VertexNormal = v.vNormal.xyz;						// short4n

	float3 direction = normalize( LightDirection );
	
	Out.vTexCoord0.x = v.vNormal.w;
	Out.vTexCoord0.z = max( dot( VertexNormal, -direction ), 0.5f );
	Out.vTexCoord0.z *= v.vNormal.w;
	Out.vNormal = v.vNormal;
	
	float2 TerrainCoord = Position.xz;
	TerrainCoord += 0.5;
	TerrainCoord /= TILE;
	Out.vTerrainTexCoord.xy  = TerrainCoord;
	
	Out.vFOWCoords.x = Position.x / 1960.0 + 0.5 / FOW_SIZE;
	Out.vFOWCoords.y = Position.z / 1228.0 + 0.5 / FOW_SIZE;

	return Out;
}

float4 PixelShader_Map2_0_General( VS_OUTPUT v ) : COLOR
{
	float FOW  = tex2D( FOWTexture, v.vFOWCoords ).b;
	float4 BaseColor = tex2D( BaseTexture, v.vTerrainTexCoord.xy );
	float vColor = tex2D( StripesTexture, v.vTerrainTexCoord.xy*1.7 ).a;
	float4 Color1 = float4(CountryColor1.rgb, 1.0) - 0.7;
	float4 Color2 = float4(CountryColor2.rgb, 1.0) - 0.7;

	float4 Color = Color2 * vColor + Color1 * ( 1.0 - vColor );

	float4 OutColor = BaseColor;

	OutColor.rgb *= v.vTexCoord0.z;

	float Grey = dot( OutColor.rgb, float3( 0.212671f, 0.715160f, 0.072169f ) );
	OutColor.rgb = Grey.rrr;

	OutColor.rgb += Color.rgb;
	OutColor.a = 1;

	OutColor.rgb += BRIGHTNESS;
	OutColor.rgb -= 0.5;
	OutColor.rgb *= CONTRAST;
	OutColor.rgb += 0.5;
	
	return float4( ApplyFOWColor(OutColor.rgb, FOW), 1.0 );
}


float4 PixelShader_Map2_0_Highlight( VS_OUTPUT v ) : COLOR
{
	float4 OutColor = float4( vAlpha, vAlpha, vAlpha, vAlpha );
	return OutColor;
}


/////////////////////////////////////////////////////////////////////////////////

VS_BORDER_OUTPUT VertexShader_Map_Border(const VS_BORDER_INPUT v )
{
	VS_BORDER_OUTPUT Out = (VS_BORDER_OUTPUT)0;

	float4 Position = float4( v.vPosition, 1 );
	Position.y = Position.y + 0.2f;
	Position.z = Position.z - 0.8f;
	Position.x = Position.x - 0.8f;

	float4x4 WorldView = ViewMatrix;
	float3 P = mul( Position, (float4x3)WorldView);
	Out.vPosition  = mul(float4(P, 1), ProjectionMatrix);

	Out.vBorderIndex = v.vBorderIndex;
	Out.vStrength = v.vStrength;
	Out.vCorner = v.vCorner;
	
	Out.vFOWCoords.x = v.vPosition.x / 1960.0 + 0.5 / FOW_SIZE;
	Out.vFOWCoords.y = v.vPosition.z / 1228.0 + 0.5 / FOW_SIZE;

	return Out;
}

float4 PixelShader_Map2_0_Border( VS_BORDER_OUTPUT v ) : COLOR
{
	float FOW  = tex2D( FOWTexture, v.vFOWCoords ).b;
	float4 BorderCords = tex2D( GeneralTexture, v.vBorderIndex );
	float4 CornerCords = tex2D(  GeneralTexture2, v.vCorner );

	float U = BorderCords.r + v.vStrength.x + CornerCords.r;
	float V = v.vStrength.y + CornerCords.g;

	if( vHeight > 100 )
		U = U + 0.5f;
	
	float2 borderTex = float2( U, V );
	
	float4 OutColor = tex2D( BorderTexture, borderTex );

	return float4( ApplyFOWColor(OutColor.rgb, FOW), OutColor.a );
}

//////////////////////////////////////////////////////////////////////////////

VS_OUTPUT_WAVES VertexShader_WaterBorder(const VS_INPUT_WAVES v )
{
	VS_OUTPUT_WAVES Out = (VS_OUTPUT_WAVES)0;

	float4 Position = float4( v.vPosition, 1 );

	float4x4 WorldView = ViewMatrix;
	float3 P = mul( Position, (float4x3)WorldView);
	Out.vPosition  = mul(float4(P, 1), ProjectionMatrix);

	Out.vWaveUV = v.vWaveData;
	
	Out.vFOWCoords.x = v.vPosition.x / 1960.0 + 0.5 / FOW_SIZE;
	Out.vFOWCoords.y = v.vPosition.z / 1228.0 + 0.5 / FOW_SIZE;

	return Out;
}

float4 PixelShader_WaterBorder( VS_OUTPUT_WAVES v ) : COLOR
{
	float FOW  = tex2D( FOWTexture, v.vFOWCoords ).b;
	float3 OutColor = float3(0.0f, 0.5f, 0.5f);
	return float4( ApplyFOWColor(OutColor, FOW), 0.5 );
}

///////////////////////////////////////////////////////////////////////////////////////////


VS_OUTPUT_WAVES VertexShader_Waves(const VS_INPUT_WAVES v )
{
	vTime = 0.6f;
	
	VS_OUTPUT_WAVES Out = (VS_OUTPUT_WAVES)0;

	float4 Position = float4( v.vPosition, 1 );
	Position.y += 0.01;

	float4x4 WorldView = ViewMatrix;
	float3 P = mul( Position, (float4x3)WorldView);
	Out.vPosition  = mul(float4(P, 1), ProjectionMatrix);

	float2 wave = v.vWaveData;
	wave.x -= 1 * vTime;
	Out.vWaveUV = wave;

	float2 waveAlpha = v.vWaveData.x;
	waveAlpha.x *= waveAlpha.y;
	Out.vWaveData = waveAlpha;
	
	Out.vFOWCoords.x = v.vPosition.x / 1960.0 + 0.5 / FOW_SIZE;
	Out.vFOWCoords.y = v.vPosition.z / 1228.0 + 0.5 / FOW_SIZE;
	
	return Out;
}


float4 PixelShader_Waves( VS_OUTPUT_WAVES v ) : COLOR
{
	float4 OutColor = tex2D( WavesTexture, v.vWaveUV );
	float FOW  = tex2D( FOWTexture, v.vFOWCoords ).b;
	OutColor.a *= v.vWaveData.x;
	return float4( ApplyFOWColor(OutColor.rgb, FOW), OutColor.a );
}

///////////////////////////////////////////////////////////////////////////////////////////

VS_OUTPUT VertexShader_Map(const VS_MAP_INPUT v )
{
	VS_OUTPUT Out = (VS_OUTPUT)0;

	float4 Position = float4( v.vPosition, 1 );

	float4x4 WorldView = ViewMatrix; // mul(WorldMatrix, ViewMatrix);
	float3 P = mul( Position, (float4x3)WorldView);
	Out.vPosition  = mul(float4(P, 1), ProjectionMatrix);
	
	Out.vColorTexCoord = v.vSmallTextureTexCoord;

	float3 VertexNormal = v.vNormal.xyz;						// short4n

	Out.vLightDirection = normalize( LightDirection );
	Out.vNormal = normalize(v.vNormal);

	float4 WorldPosition = float4( v.vPosition, 1);

	float2 TerrainCoord = WorldPosition.xz;
	TerrainCoord += 0.5;
	TerrainCoord /= TILE;
	Out.vTerrainTexCoord.xy  = TerrainCoord;

	Out.vTerrainTexCoord.z = saturate( ( 0.95 - VertexNormal.y ) * 5.0 );
	Out.vFOWCoords.x = v.vPosition.x / 1960.0 + 0.5 / FOW_SIZE;
	Out.vFOWCoords.y = v.vPosition.z / 1228.0 + 0.5 / FOW_SIZE;
	return Out;
}


float4 PixelShader_Map2_0( VS_OUTPUT v ) : COLOR
{
	float4 Color = tex2D( ColorTexture, v.vColorTexCoord );
	float FOW  = tex2D( FOWTexture, v.vFOWCoords ).b;
		
	float4 OutColor = Color;
	float vDetail = 0;

	float NdotL = max(0, dot(normalize(v.vNormal), -normalize(v.vLightDirection)));
	OutColor.rgb *= NdotL;

	OutColor.rgb += BRIGHTNESS;
	OutColor.rgb -= 0.5f;
	OutColor.rgb *= CONTRAST;
	OutColor.rgb += 0.5f;
	
	OutColor.rgb += (1.0f - NdotL) * AMBIENT;

	return float4( ApplyFOWColor(OutColor.rgb, FOW), OutColor.a );
}


///////////////////////////////////////////////////////////////////////////////////////////

VS_OUTPUT_PTI VertexShader_PTI(const VS_INPUT_PTI v )
{
	VS_OUTPUT_PTI Out = (VS_OUTPUT_PTI)0;

	float3 P = mul( v.vPosition, (float4x3)ViewMatrix);
	Out.vPosition  = mul(float4(P, 1), ProjectionMatrix);
	return Out;
}

float4 PixelShader_PTI( VS_OUTPUT_PTI v ) : COLOR
{
	return float4( 0, 0, 0, 1 );
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////

VS_TREE_OUTPUT VertexShader_Trees(const VS_TREE_INPUT v)
{
	VS_TREE_OUTPUT Out = (VS_TREE_OUTPUT)0;

	
	float4 Position = float4( v.vPosition, 1 );
	float4x4 WorldView = ViewMatrix;
	float3 P = mul( Position, (float4x3)WorldView);
	Out.vPosition  = mul(float4(P, 1), ProjectionMatrix);
	Out.vTexCoord0 = v.vTexCoord0;
	
	Out.vFOWCoords.x = v.vPosition.x / 1960.0 + 0.5 / FOW_SIZE;
	Out.vFOWCoords.y = v.vPosition.z / 1228.0 + 0.5 / FOW_SIZE;
	
	return Out;
}

float4 PixelShader_Trees(VS_TREE_OUTPUT v) : COLOR
{
	float FOW  = tex2D( FOWTexture, v.vFOWCoords ).b;
	float4 Color = tex2D( TreeTexture, v.vTexCoord0 );
	return float4( ApplyFOWColor( Color.rgb, FOW ), Color.a);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////

// in-game terrain shader
technique TerrainShader_Graphical
{
	pass p0
	{
		VertexShader = compile vs_2_0 VertexShader_Map();
		PixelShader = compile ps_2_0 PixelShader_Map2_0();
	}
}

technique TerrainShader_General
{
	pass p0
	{
		VertexShader = compile vs_2_0 VertexShader_Map_General();
		PixelShader = compile ps_2_0 PixelShader_Map2_0_General();
	}
}

technique TerrainShader_Highlight
{
	pass p0
	{
		VertexShader = compile vs_2_0 VertexShader_Map_General();
		PixelShader = compile ps_2_0 PixelShader_Map2_0_Highlight();
	}
}

technique TerrainShader_Border
{
	pass p0
	{
		VertexShader = compile vs_2_0 VertexShader_Map_Border();
		PixelShader = compile ps_2_0 PixelShader_Map2_0_Border();
	}
}

technique TerrainShader_WaterBorder
{
	pass p0
	{
		VertexShader = compile vs_2_0 VertexShader_WaterBorder();
		PixelShader = compile ps_2_0 PixelShader_WaterBorder();
	}
}

technique WavesShader
{
	pass p0
	{
		VertexShader = compile vs_2_0 VertexShader_Waves();
		PixelShader = compile ps_2_0 PixelShader_Waves();
	}
}


technique PTIShader
{
	pass p0
	{
		fvf = XYZ;

		LightEnable[0] = false;
		Lighting = False;

		ALPHABLENDENABLE = True;

		ColorOp[0] = Modulate;
		ColorArg1[0] = Texture;
		ColorArg2[0] = current;
  
		ColorOp[1] = Disable;
		AlphaOp[1] = Disable;

		VertexShader = compile vs_2_0 VertexShader_PTI();
		PixelShader = compile ps_2_0 PixelShader_PTI();
	}
}

technique TreeShader
{
	pass p0
	{
		AlphaBlendEnable = True;
		
		ALPHAREF = 1;
		ALPHATESTENABLE = True; 
	    AlphaFunc = GreaterEqual;
		
		SrcBlend  = One;
		DestBlend = InvSrcAlpha;
	  
		ColorOp[0] = Modulate;
		ColorArg1[0] = Texture;
		ColorArg2[0] = Current;
		
		AlphaOp[0] = SelectArg1;
		AlphaArg1[0] = Texture;
		AlphaArg2[0] = Current;
		
		ColorOp[1] = Disable;
		AlphaOp[1] = Disable;
		
		VertexShader = compile vs_2_0 VertexShader_Trees();
		PixelShader = compile ps_2_0 PixelShader_Trees();
	}
}
